from zope.schema import Float, Int
from zope.interface import Interface
from zope.annotation.interfaces import IAnnotatable
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('wc.rating')

class IRatable(IAnnotatable):
    """Marker interface that promises that an implementing object maybe
    rated using ``IRating`` annotations.
    """

class IRating(Interface):
    """Give and query rating about objects, such as recipes.
    """

    def rate(rating):
        """Rate the current object with `rating`.
        """

    average = Float(
        title=_(u"Average rating"),
        description=_(u"The average rating of the current object"),
        required=True
        )

    numberOfRatings = Int(
        title=_(u"Number of ratings"),
        description=_(u"The number of times the current has been rated"),
        required=True
        )



from zope.viewlet.viewlet import ViewletBase
from zope.app.pagetemplate import ViewPageTemplateFile
from wc.rating.interfaces import IRating

class RatingViewlet(ViewletBase):

    ratingChoices = (1, 2, 3, 4, 5)

    def update(self):
        self.rating = IRating(self.context)
        userinput = self.request.form.get('wc.rating')
        if userinput is not None:
            self.rating.rate(userinput)

    render = ViewPageTemplateFile('viewlet.pt')



import unittest
from doctest import DocFileSuite

import zope.component.testing
from zope.annotation.attribute import AttributeAnnotations
from wc.rating.rating import rating_adapter

def setUp(test):
    zope.component.testing.setUp(test)
    zope.component.provideAdapter(AttributeAnnotations)
    zope.component.provideAdapter(rating_adapter)

def test_suite():
    return unittest.TestSuite((
        DocFileSuite('README.txt',
                     package='wc.rating',
                     setUp=setUp,
                     tearDown=zope.component.testing.tearDown),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from setuptools import setup, find_packages

setup(name='wc.rating',
      version='3.9',
      url='http://worldcookery.com/Downloads',
      description='Rating functionality for Zope content objects',
      author='Philipp von Weitershausen',
      author_email='philipp@weitershausen.de',
      long_description=file('README.txt').read(),
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Environment :: Web Environment',
                   'Framework :: Zope3',
                   'Intended Audience :: Developers'],

      packages=find_packages(),
      namespace_packages=['wc',],
      install_requires=['setuptools',
                        'ZODB3',
                        'zope.interface',
                        'zope.schema',
                        'zope.component',
                        'zope.annotation',
                        'zope.i18nmessageid',
                        # wc.worldcookeryskin viewlet support
                        'zope.viewlet',
                        'zope.app.pagetemplate',
                        'wc.worldcookeryskin',
                        ],
      include_package_data=True,
      zip_safe=False,
      )



import zope.component
import zope.interface
import persistent.list
from zope.annotation import factory
from wc.rating.interfaces import IRating, IRatable

class Rating(persistent.Persistent):
    zope.interface.implements(IRating)
    zope.component.adapts(IRatable)

    def __init__(self):
        self.average = 0.0
        self.numberOfRatings = 0
        self.ratings = persistent.list.PersistentList()

    def rate(self, rating):
        ratings = self.ratings
        ratings.append(float(rating))
        self.numberOfRatings += 1
        self.average = sum(ratings)/len(ratings)

rating_adapter = factory(Rating)
